
function Test1(hdr_rate,pam_levels,cable_name,pcb_cutoff,print_plot)
% Simulation of ACT transmit and receive signals - Test 1
% Function arguments:
%   Test1(hdr_rate,pam_levels,cable_name,pcb_cutoff,print_plot)
% where hdr_rate is the data rate in the high data rate direction, given 
% in Gbps (default is 2.5, for 2.5Gbps). The pam_levels is the number of 
% levels in the high data rate PAM modulation (default is 4, for PAM4). 
% The cable_name parameter identifies the cable model to use in the 
% simulation. The pcb_cutoff parameter gives the cutoff frequency, 
% in MHz, for the high-pass filter used to model the high-pass nature of 
% the PoC circuitry on the PCB. The unit for this parameter is MHz
% and the default value is 10, for 10MHz. The print_plot parameter 
% controls if the the plots generated by the test code should be saved 
%to an image file (default is 0, for not saving the image files).
%
% This is simulation code provided to help with the development of
% IEEE 802.3dm.
%
% This code is provided for reference to allow independent evaluation
% of the accuracy and applicability of the simulation results shared in
% IEEE 802.3dm presentations by the author.
%
% Written by Ragnar Jonsson, affiliated with Marvell Technology, Inc.
% Version 1.1, January 20th, 2025
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.

%%% test if MATLAB or Octave %%%
is_octave = exist('OCTAVE_VERSION', 'builtin') ~= 0;

%%% Default arguments %%%
if((nargin < 1) || isempty(hdr_rate))
    hdr_rate = 2.5; % Set default high data rate to 2.5 Gbps
end
if((nargin < 2) || isempty(pam_levels))
    pam_levels = 4; % Set default PAM levels to 4 (PAM4)
end
if((nargin < 3) || isempty(cable_name))
    cable_name = 'good'; % Set default cable model to 'good'
end
if((nargin < 4) || isempty(pcb_cutoff))
    pcb_cutoff = 10; % Set default PCB cutoff frequency to 10 MHz
end
if((nargin < 5) || isempty(print_plot))
    print_plot = 0; % Set default to not save the plots
end

%%% Check data rate %%%
switch(hdr_rate)
    case 2.5
        hdr_rate_text = '2.5'; % Set text for 2.5 Gbps
    case 5
        hdr_rate_text = '5'; % Set text for 5 Gbps
    case 10
        hdr_rate_text = '10'; % Set text for 10 Gbps
    otherwise
        error(sprintf('Invalid data rate selected %2.2fGbps',hdr_rate)); % Error for invalid data rate
end

%%% Load cable impulse response %%%
switch(cable_name)
    case 'good'
        cable_file = 'cable_jonsson_3dm_02_09_15_24_good.mat'; % Load 'good' cable model
    case 'bad'
        cable_file = 'cable_jonsson_3dm_02_09_15_24_bad.mat'; % Load 'bad' cable model
    otherwise
        cable_file = cable_name; % Load custom cable model
        disp('Warning: loading untested cable file'); % Display warning for untested cable file
end
load(cable_file) % Load the cable file
is_coax = cable.is_coax; % Check if the cable is coaxial

%%% Load hybrid model %%%
load jonsson_e241216b_hybrid_echo.mat % Load hybrid echo model

%%% Configure simulation variables %%%
ldr_symbols = 100; % Number of low data rate symbols
t = [0:(192*ldr_symbols)-1]/cable.F_s; % Time vector for simulation

%%% Create test case name %%%
if(is_coax)
    cable_type = 'coax'; % Set cable type to coax
    cable_type_indicator = 'V1'; % Set cable type indicator for coax
else
    cable_type = 'stp'; % Set cable type to STP
    cable_type_indicator = 'T1'; % Set cable type indicator for STP
end
test_config = sprintf('%sG_%s_%s',hdr_rate_text,cable_name,cable_type); % Create test configuration name
camera_text = sprintf('%sG+100MBASE-%s',hdr_rate_text,cable_type_indicator); % Create camera text
host_text = sprintf('100M+%sGBASE-%s',hdr_rate_text,cable_type_indicator); % Create host text

%%% Configure the PCB impairments %%%
pcb_config.hybrid_echo = h_hybrid_echo; % Set hybrid echo for PCB
pcb_config.cutoff = pcb_cutoff; % Set PCB cutoff frequency
pcb_config.noise = '0;'; % Set PCB noise to zero

%%% No AFE internal noise in this simulation %%%
afe_config.dut.noise = 'state.out = 0.000*randn(size(x));'; % Set DUT AFE noise to zero
afe_config.lkp.noise = 'state.out = 0.000*randn(size(x));'; % Set LKP AFE noise to zero

%%% Configure equalizer lengths %%%
eq_config.dut.ffe_length = 1; % Set DUT FFE length
eq_config.dut.dfe_length = 1; % Set DUT DFE length
eq_config.lkp.ffe_length = 30; % Set LKP FFE length
eq_config.lkp.dfe_length = 10; % Set LKP DFE length

%%% Run test case %%%
act = simDM_ACT_test(hdr_rate,pam_levels,cable,pcb_config,afe_config,eq_config); % Run the test case
  
%%% Plot the signal eye-diagrams %%%
signal_list = {
    'Camera AFE transmit',act.dut.afe.tx_out,act.hdr_oversampling,5,camera_text;
    'Camera MDI transmit',act.dut.pcb.tx_out,act.hdr_oversampling,3,camera_text;
    'Camera MDI receive',act.channel.port2_out,act.ldr_oversampling,4,host_text;
    'Camera PCB receive',act.dut.pcb.rx_out,act.ldr_oversampling,6,host_text;
    'Camera AFE receive',act.dut.afe.rx_out,act.ldr_oversampling,7,host_text;
    'Host AFE transmit',act.lkp.afe.tx_out,act.ldr_oversampling,5,host_text;
    'Host MDI transmit',act.lkp.pcb.tx_out,act.ldr_oversampling,2,host_text;
    'Host MDI receive',act.channel.port1_out,act.hdr_oversampling,1,camera_text;
    'Host PCB receive',act.lkp.pcb.rx_out,act.hdr_oversampling,6,camera_text;
    'Host AFE receive',act.lkp.afe.rx_out,act.hdr_oversampling,5,camera_text;
};
N_signals = size(signal_list,1); % Get the number of signals
for n=1:N_signals
    figure
    signal_name = signal_list{n,1}; % Get signal name
    signal = signal_list{n,2}; % Get signal data
    oversampling = signal_list{n,3}; % Get oversampling factor
    signal_max = max(signal); % Get maximum signal value
    signal_power = 10*log10(var(signal)/50*1e3); % Calculate signal power in dB
    simDM_plot_eye(signal(1:oversampling*100)*1e3,t(1:oversampling*2)*1e9); % Plot eye-diagram
    xlabel('Time [ns]') % Set x-axis label
    ylabel('Signal Level [mV]') % Set y-axis label
    axis([0 t(oversampling*2)*1e9 -800 800]) % Set axis limits
    text(0.1,700,sprintf('Average Power = %2.1fdB',signal_power)) % Display average power
    grid on
    probe_text = sprintf('(%d) - %s',signal_list{n,4},signal_list{n,1}); % Create probe text
    function_text = sprintf('Test1(%s,%d,%s)',hdr_rate_text,pam_levels,cable_name); % Create function text
    text(t(oversampling*2)*1e9*0.01,-800*0.85,{probe_text,host_text,function_text},'BackgroundColor',[1,1,1]*0.95,'interpreter','none') % Display text on plot
    title(sprintf('%s eye-diagram with HDR %2.1fGbps PAM%d',signal_name,hdr_rate,pam_levels)); % Set plot title
    if(print_plot)
      img_file = sprintf('%s_%s_eye_pam%d_%s.jpg',mfilename(),signal_name,pam_levels,test_config); % Create image file name
      img_file = strrep(lower(img_file),' ','_'); % Replace spaces with underscores in file name
      if(is_octave)
        print(img_file) % Save plot to file
      else
        saveas(gcf,img_file,'jpg') % Save plot to file
      end
    end
end
  
%%% Plot the HDR "eye-diagram" for each test run %%%
figure
signal_name = 'slicer input'; % Set signal name
plot(t(1:act.hdr_oversampling:end)*1e9,act.lkp.pma.slicer_in,'.') % Plot slicer input
xlabel('Time [ns]') % Set x-axis label
ylabel('Signal Level') % Set y-axis label
grid on
axis([0 t(end)*1e9 -1.5 1.5]) % Set axis limits
function_text = sprintf('Test1(%s,%d,%s)',hdr_rate_text,pam_levels,cable_name); % Create function text
text(t(end)*1e9*0.01,-1.5*0.85,{'(A) - Host Slicer Input',host_text,function_text},'BackgroundColor',[1,1,1]*0.95,'interpreter','none') % Display text on plot
title(sprintf('Host slicer input with HDR %2.1fGbps PAM%d',hdr_rate,pam_levels)); % Set plot title
if(print_plot)
  img_file = sprintf('%s_%s_eye_pam%d_%s.jpg',mfilename(),signal_name,pam_levels,test_config); % Create image file name
  img_file = strrep(lower(img_file),' ','_'); % Replace spaces with underscores in file name
  if(is_octave)
    print(img_file) % Save plot to file
  else
    saveas(gcf,img_file,'jpg') % Save plot to file
  end
end

end

