function Test4(hdr_rate,pam_levels,cable_name,env_noise,pcb_cutoff,print_plot)
% Simulation of ACT transmit and receive signals - Test 4
%   Test4(hdr_rate,pam_levels,cable_name,env_noise,pcb_cutoff,print_plot)
% Function arguments:
%   hdr_rate   - Data rate in Gbps (default: 2.5)
%   pam_levels - Number of PAM levels (default: 4)
%   cable_name - Cable model name (default: 'good')
%   env_noise  - Environmental noise (default: '0')
%   pcb_cutoff - PCB cutoff frequency in MHz (default: 10)
%   print_plot - Flag to save plots (default: 0)
%
% This is simulation code provided to help with the development of
% IEEE 802.3dm.
%
% This code is provided for reference to allow independent evaluation
% of the accuracy and applicability of the simulation results shared in
% IEEE 802.3dm presentations by the author.
%
% Written by Ragnar Jonsson, affiliated with Marvell Technology, Inc.
% Version 1.1.1, February 24th, 2025
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.

%%% test if MATLAB or Octave %%%
is_octave = exist('OCTAVE_VERSION', 'builtin') ~= 0;

%%% Default arguments %%%
if((nargin < 1) || isempty(hdr_rate))
    hdr_rate = 2.5; % Set default high data rate to 2.5 Gbps
end
if((nargin < 2) || isempty(pam_levels))
    pam_levels = 4; % Set default PAM levels to 4 (PAM4)
end
if((nargin < 3) || isempty(cable_name))
    cable_name = 'good'; % Set default cable model to 'good'
end
if((nargin < 4) || isempty(env_noise))
    env_noise = '0;'; % Set default environmental noise to '0'
end
if((nargin < 5) || isempty(pcb_cutoff))
    pcb_cutoff = 10; % Set default PCB cutoff frequency to 10 MHz
end
if((nargin < 6) || isempty(print_plot))
    print_plot = 0; % Set default to not save the plots
end

%%% Check data rate %%%
switch(hdr_rate)
    case 2.5
        hdr_rate_text = '2.5'; % Set text for 2.5 Gbps
    case 5
        hdr_rate_text = '5'; % Set text for 5 Gbps
    case 10
        hdr_rate_text = '10'; % Set text for 10 Gbps
    otherwise
        error(sprintf('Invalid data rate selected %2.2fGbps',hdr_rate)); % Error for invalid data rate
end

%%% test cable list %%%
test_cable = {
    'jonsson_3dm_01_03_10_25_simDM_cables/cable_jonsson_dm_cx174(7)_s2p.mat',                                                                                           
    'jonsson_3dm_01_03_10_25_simDM_cables/cable_jonsson_dm_cx31(15)_s2p.mat'                                                                                           
    'jonsson_3dm_01_03_10_25_simDM_cables/cable_jonsson_dm_cx174(2_5)~c~cx31(6)_s2p.mat',                                                                               
    'jonsson_3dm_01_03_10_25_simDM_cables/cable_jonsson_dm_cx174(2_5)~c~cx31(5)~c~cx31(5)_s2p.mat',                                                                    
    'jonsson_3dm_01_03_10_25_simDM_cables/cable_jonsson_dm_cx174(0_3)~c~cx174(0_36)~c~cx31(1)~c~cx31(11_84)~c~cx31(1_5)_s2p.mat',                                    
    'jonsson_3dm_01_03_10_25_simDM_cables/cable_jonsson_dm_cx174(0_3,47)~c(bad)~cx174(0_36,53)~c(bad)~cx31(6_34,47)~c(bad)~cx31(4,53)~c(bad)~cx31(4,47)_s2p.mat',       
    'jonsson_3dm_01_03_10_25_simDM_cables/cable_jonsson_dm_cx174(0_3,47)~c(ugly)~cx174(0_36,53)~c(ugly)~cx31(1,47)~c(ugly)~cx31(11_84,53)~c(ugly)~cx31(1_5,47)_s2p.mat',
    'jonsson_3dm_01_03_10_25_simDM_cables/cable_jonsson_dm_cx174(0_3,47)~c(good)~cx174(0_36,53)~c(good)~cx31(1,47)~c(good)~cx31(11_84,53)~c(good)~cx31(1_5,47)_s2p.mat',
    'jonsson_3dm_01_03_10_25_simDM_cables/cable_jonsson_dm_cx174(1)~c~cx31(10_4)_s2p.mat',                                                                              
    'jonsson_3dm_01_03_10_25_simDM_cables/cable_jonsson_dm_cx174(3)~c~cx174(3)~c~cx31(3)~c~cx31(3)~c~cx31(3)_s2p.mat',                                                  
};

%%% Load cable impulse response %%%
switch(cable_name)
    case 'good'
        cable_file = 'cable_jonsson_3dm_02_09_15_24_good.mat'; % Load 'good' cable model
    case 'bad'
        cable_file = 'cable_jonsson_3dm_02_09_15_24_bad.mat'; % Load 'bad' cable model
    otherwise
        if(regexp(cable_name,'cable([0123456789]+)') == 1)
            ix = regexp(cable_name,'cable([0123456789]+)');
            cable_file = test_cable{ix};
        else
            cable_file = cable_name; % Load custom cable model
            disp('Warning: loading untested cable file'); % Display warning for untested cable file
        end
end
load(cable_file) % Load the cable file
is_coax = cable.is_coax; % Check if the cable is coaxial

%%% Load hybrid model %%%
load jonsson_e241216b_hybrid_echo.mat % Load hybrid echo model

%%% Configure simulation variables %%%
ldr_symbols = 100; % Number of low data rate symbols
t = [0:(192*ldr_symbols)-1]/cable.F_s; % Time vector for simulation

%%% Create test case name %%%
if(is_coax)
    cable_type = 'coax'; % Set cable type to coax
    cable_type_indicator = 'V1'; % Set cable type indicator for coax
else
    cable_type = 'stp'; % Set cable type to STP
    cable_type_indicator = 'T1'; % Set cable type indicator for STP
end
test_config = sprintf('%sG_%s_%s',hdr_rate_text,cable_name,cable_type); % Create test configuration name
camera_text = sprintf('%sG+100MBASE-%s',hdr_rate_text,cable_type_indicator); % Create camera text
host_text = sprintf('100M+%sGBASE-%s',hdr_rate_text,cable_type_indicator); % Create host text

%%% Configure the PCB impairments %%%
pcb_config.hybrid_echo = h_hybrid_echo; % Set hybrid echo for PCB
pcb_config.cutoff = pcb_cutoff; % Set PCB cutoff frequency
pcb_config.noise = env_noise; % Set environmental noise
env_noise_name = regexp(env_noise,'modulated_pulse(.*)','match'); %%% shorter name for noise
if(length(env_noise_name) > 0)
    env_noise_name = env_noise_name{1}(1:end-1);
else
    env_noise_name = env_noise;
end

%%% No AFE internal noise in this simulation %%%
afe_config.dut.noise = 'state.out = 0.000*randn(size(x));'; % Set DUT AFE noise to zero
afe_config.lkp.noise = 'state.out = 0.000*randn(size(x));'; % Set LKP AFE noise to zero

%%% Configure equalizer lengths %%%
eq_config.dut.ffe_length = 1; % Set DUT FFE length
eq_config.dut.dfe_length = 1; % Set DUT DFE length
eq_config.lkp.ffe_length = 30; % Set LKP FFE length
eq_config.lkp.dfe_length = 10; % Set LKP DFE length

%%% Run test case %%%
act = simDM_ACT_test(hdr_rate,pam_levels,cable,pcb_config,afe_config,eq_config); % Run the test case
  
%%% Plot the signal eye-diagrams %%%
signal_list = {
    %'Camera AFE transmit',act.dut.afe.tx_out,act.hdr_oversampling,5,camera_text;
    %'Camera MDI transmit',act.dut.pcb.tx_out,act.hdr_oversampling,3,camera_text;
    %'Camera MDI receive',act.channel.port2_out,act.ldr_oversampling,4,host_text;
    %'Camera PCB receive',act.dut.pcb.rx_out,act.ldr_oversampling,6,host_text;
    'Camera AFE receive',act.dut.afe.rx_out,act.ldr_oversampling,7,host_text;
    %'Host AFE transmit',act.lkp.afe.tx_out,act.ldr_oversampling,5,host_text;
    %'Host MDI transmit',act.lkp.pcb.tx_out,act.ldr_oversampling,2,host_text;
    %'Host MDI receive',act.channel.port1_out,act.hdr_oversampling,1,camera_text;
    %'Host PCB receive',act.lkp.pcb.rx_out,act.hdr_oversampling,6,camera_text;
    %'Host AFE receive',act.lkp.afe.rx_out,act.hdr_oversampling,5,camera_text;
};
N_signals = size(signal_list,1); % Get the number of signals
for n=1:N_signals
    figure
    signal_name = signal_list{n,1}; % Get signal name
    signal = signal_list{n,2}; % Get signal data
    oversampling = signal_list{n,3}; % Get oversampling factor
    signal_max = max(signal); % Get maximum signal value
    signal_power = 10*log10(var(signal)/50*1e3); % Calculate signal power in dB
    simDM_plot_eye(signal(1:oversampling*100)*1e3,t(1:oversampling*2)*1e9); % Plot eye-diagram
    xlabel('Time [ns]') % Set x-axis label
    ylabel('Signal Level [mV]') % Set y-axis label
    axis([0 t(oversampling*2)*1e9 -800 800]) % Set axis limits
    text(0.1,700,sprintf('Average Power = %2.1fdB',signal_power)) % Display average power
    grid on
    probe_text = sprintf('(%d) - %s  -  %s',signal_list{n,4},signal_list{n,1},host_text); % Create probe text
    function_text = sprintf('Test4(%s,%d,%s,%s)',hdr_rate_text,pam_levels,cable_name,env_noise_name); % Create function text
    text(t(oversampling*2)*1e9*0.01,-800*0.9,{probe_text,function_text},'BackgroundColor',[1,1,1]*0.95,'interpreter','none') % Display text on plot
    title(sprintf('%s eye-diagram with HDR %2.1fGbps PAM%d',signal_name,hdr_rate,pam_levels)); % Set plot title
    if(print_plot)
      img_file = sprintf('%s_%s_eye_pam%d_%s_%s.jpg',mfilename(),signal_name,pam_levels,test_config,env_noise_name); % Create image file name
      img_file = strrep(lower(img_file),' ','_'); % Replace spaces with underscores in file name
      if(is_octave)
        print(img_file) % Save plot to file
      else
        saveas(gcf,img_file,'jpg') % Save plot to file
      end
    end
end
  
%%% Plot the HDR "eye-diagram" for each test run %%%
figure
signal_name = 'slicer input'; % Set signal name
plot(t(1:act.hdr_oversampling:end)*1e9,act.lkp.pma.slicer_in,'.') % Plot slicer input
hold on
plot([0 1]*t(end)*1e9,[0 0],'r','linewidth',2)
if(pam_levels == 4)
    plot([0 1]*t(end)*1e9,[1 1]*2/3,'r','linewidth',2)
    plot([0 1]*t(end)*1e9,-[1 1]*2/3,'r','linewidth',2)
end
hold off
xlabel('Time [ns]') % Set x-axis label
ylabel('Signal Level') % Set y-axis label
grid on
axis([0 t(end)*1e9 -1.5 1.5]) % Set axis limits
function_text = sprintf('Test4(%s,%d,%s,%s)',hdr_rate_text,pam_levels,cable_name,env_noise_name); % Create function text
text(t(end)*1e9*0.01,-1.5*0.9,{['(A) - Host Slicer Input  -  ',host_text],function_text},'BackgroundColor',[1,1,1]*0.95,'interpreter','none') % Display text on plot
title(sprintf('Host slicer input with HDR %2.1fGbps PAM%d',hdr_rate,pam_levels)); % Set plot title
if(print_plot)
  img_file = sprintf('%s_%s_eye_pam%d_%s_%s.jpg',mfilename(),signal_name,pam_levels,test_config,env_noise_name); % Create image file name
  img_file = strrep(lower(img_file),' ','_'); % Replace spaces with underscores in file name
  if(is_octave)
    print(img_file) % Save plot to file
  else
    saveas(gcf,img_file,'jpg') % Save plot to file
  end
end

figure()
dx = 0.02
[p,x] = hist(act.lkp.pma.slicer_in,[-1.5:dx:1.5]);
p = p/sum(p)/dx;
sum(p)*dx
area(x,p)
hold on
plot([0 0],[0 6],'r','linewidth',2)
if(pam_levels == 4)
    plot([1 1]*2/3,[0 6],'r','linewidth',2)
    plot(-[1 1]*2/3,[0 6],'r','linewidth',2)
end
hold off
grid on
axis([-1.5 1.5 0 6])
xlabel('Slicer Input Level')
ylabel('Probability Density Function (PDF)')
title(sprintf('Host slicer input with HDR %2.1fGbps PAM%d',hdr_rate,pam_levels)); % Set plot title
text(-1.4,2.35/2.5*6,{['(A) - Host Slicer Input  -  ',host_text],function_text},'BackgroundColor',[1,1,1]*0.95,'interpreter','none') % Display text on plot
if(print_plot)
  img_file = sprintf('%s_%s_hist_pam%d_%s_%s.jpg',mfilename(),signal_name,pam_levels,test_config,env_noise_name); % Create image file name
  img_file = strrep(lower(img_file),' ','_'); % Replace spaces with underscores in file name
  if(is_octave)
    print(img_file) % Save plot to file
  else
    saveas(gcf,img_file,'jpg') % Save plot to file
  end
end

if(0)
% Plot Camera PCB noise
figure;
plot(t * 1e9, act.dut.pcb.noise.out * 1e3);
title('Camera PCB noise');
xlabel('Time [ns]'); % Set x-axis label
ylabel('Signal Level [mV]'); % Set y-axis label
grid on;
axis([0, t(end) * 1e9, -1000, 1000]); % Set axis limits
text(t(end) * 1e9 * 0.05, -700, {'Environmental Noise Command:', env_noise}, 'BackgroundColor', [0.95, 0.95, 0.95], 'Interpreter', 'none'); % Display environmental noise text on plot

% Plot Host PCB noise
figure;
plot(t * 1e9, act.lkp.pcb.noise.out * 1e3);
title('Host PCB noise');
xlabel('Time [ns]'); % Set x-axis label
ylabel('Signal Level [mV]'); % Set y-axis label
grid on;
axis([0, t(end) * 1e9, -1000, 1000]); % Set axis limits
text(t(end) * 1e9 * 0.05, -700, {'Environmental Noise Command:', env_noise}, 'BackgroundColor', [0.95, 0.95, 0.95], 'Interpreter', 'none'); % Display environmental noise text on plot
end

end

