function [ffe,dfe,err,RR,R_g,R_n,R_g0,g0] = mmse_eq(g,R_noise,n0,N_ffe,N_dfe,N_os,eq_type,inv_method)

%mmse_eq() Generic MMSE equalization algorithm for DFE, FFE, ZFE, & MF
%Usage:
%  [ffe,dfe,err] = mmse_eq(g,R_noise,n0,N_ffe,N_dfe,N_os,eq_type)
% where <g> is the channel impulse responce, <R_noise> is the noise
% autocorrelation function, <n0> is the delay introduced by the equalizer,
% <N_ffe> in the number of taps in the feed forward filter, <N_dfe> is the
% number of taps in the decision feedback filter, <N_os> is the oversampling
% ratio, <eq_type> is the equalizer structure type (DFE, TEQ, ZFE, MF), and
% <inv_method> is the matrix inversion method (should normaly be set to the
% defult value of zero).
% Based on algorithm presented in Chaper 11.5 of the book "Fundementals of
% DSL Technology".

% Created by Ragnar H. Jonsson
%
% Permission is hereby granted, free of charge, to any person obtaining
% a copy of this software and associated documentation files (the "Software"),
% to deal in the Software without restriction, including without limitation
% the rights to use, copy, modify, merge, publish, distribute, sublicense,
% and/or sell copies of the Software, and to permit persons to whom the
% Software is furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.

if( nargin < 2 )
  R_noise = 0; % zero forcing eq.
end;
if( nargin < 3 )
  n0 = prod(size(g))/2;
end;
if( nargin < 4 )
  N_ffe = prod(size(g));
end;
if( nargin < 5 )
  N_def = 0;  % Linear eq.
end;
if( nargin < 6 )
  N_os = 1;  % T-spaced
end;
if( nargin < 7 )
  eq_type = 'DFE';
end;
if( nargin < 8 )
  inv_method = 0;
end;

%%% initialize %%%
N_g = prod(size(g));
N_G = floor((N_g+N_ffe-1)/N_os);
G = zeros(N_G,N_ffe);
N0 = N_os*round(n0/N_os);

%%% find target structure %%%
if( ischar(eq_type) )
  switch( upper(eq_type) )
    case {'DFE','ISE','TEQ'},
      eq_type = [1 1 1];
    case 'ZFE',
      eq_type = [1 1 0];
    case 'MF',
      eq_type = [0 0 1];
    otherwise,
      error(['Unknown equalizer structure: ' eq_type]);
  end;
end;
k_g  = eq_type(1);
k_g0 = eq_type(2);
k_n  = eq_type(3);

%%% find signal auto-correlation matrices %%%
g0 = [zeros(1,N_ffe-1) g zeros(1,max(N_dfe*N_os+N0,N_ffe)-1)];
ix = N_ffe+1+[0:N_G-1]*N_os;
ix0 = N_ffe+1+[1:N_dfe]*N_os + N0;
for n=1:N_ffe,
  G(:,n) = (g0(ix-n)).';
  G_dfe(:,n) = (g0(ix0-n)).';
end;
R_g = G'*G;
R_g0 = G_dfe'*G_dfe;

%%% find noise auto-correlation matrix %%%
N_noise = prod(size(R_noise));
if( N_noise==1 )
  R_n = eye(N_ffe)*R_noise;
else
  ix_n = ones(N_ffe,1)*[0:N_ffe-1];
  if( N_noise < N_ffe )
    R_noise(N_ffe) = 0;
  end;
  R_n = R_noise(abs(ix_n-ix_n')+1);
end;

%%% find FFE coefficients %%%
g0 = g0(N_ffe + N0 - [0:N_ffe-1])';
RR = k_g*R_g - k_g0*R_g0 + k_n*R_n;
switch( inv_method )
  case 1,
    ffe = inv_method1(RR,g0); %%% user specified inversion
  case 2,
    R_inv = inv_method2(RR);  %%% user specified inversion
    ffe = R_inv*g0;
  otherwise,
    R_inv = inv(RR);
    ffe = R_inv*g0;
end;

%%% find DFE coefficients %%%
dfe = G_dfe*ffe;

%%% find error %%%
err = real(1 - 2*ffe'*g0 + ffe'*RR*ffe);

%%%%%%%%%%%%%%%%%%%% End of mmse_eq.m %%%%%%%%%%%%%%%%%%%%
