function act = simDM_ACT_test(hdr_rate, pam_levels, cable, pcb_config, afe_config, eq_config)
% simDM_ACT_test Simulates the ACT (Active Channel Termination) for IEEE 802.3dm.
% 
% This function simulates the ACT for IEEE 802.3dm using the provided
% configuration parameters for the header rate, PAM levels, cable, PCB,
% AFE, and equalizer configurations.
%
% Inputs:
%   hdr_rate   - High data rate for the simulation
%   pam_levels - Number of PAM levels
%   cable      - Struct containing cable configuration
%   pcb_config - Struct containing PCB configuration
%   afe_config - Struct containing AFE configuration
%   eq_config  - Struct containing equalizer configuration
%
% Outputs:
%   act - Struct containing the simulation results
%
% This is simulation code provided to help with the development of
% IEEE 802.3dm.
%
% This code is provided for reference to allow independent evaluation
% of the accuracy and applicability of the simulation results shared in
% IEEE 802.3dm presentations by the author.
%
% Written by Ragnar Jonsson, affiliated with Marvell Technology, Inc.
% Version 1.1, January 20th, 2025
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.

  %%% Initialize %%%
  ldr_symbols = 100; % Number of low data rate symbols

  %%% Construct ACT simDM %%%
  act = simDM_ACT(hdr_rate, cable.is_coax, pcb_config.cutoff, pam_levels); % Initialize the ACT simulation

  %%% Configure the hybrid echo filter %%%
  act.dut.pcb.echo_filter.b = pcb_config.hybrid_echo; % Set DUT hybrid echo filter coefficients
  act.lkp.pcb.echo_filter.b = pcb_config.hybrid_echo; % Set LKP hybrid echo filter coefficients

  %%% Configure the channel model %%%
  act.channel.p11.b = cable.h{1,1}; % Set channel model p11 coefficients
  act.channel.p12.b = cable.h{1,2}; % Set channel model p12 coefficients
  act.channel.p22.b = cable.h{2,2}; % Set channel model p22 coefficients
  act.channel.p21.b = cable.h{2,1}; % Set channel model p21 coefficients

  %%% Configure AFE internal noise %%%
  act.dut.afe.noise.eval = afe_config.dut.noise; % Set DUT AFE noise model
  act.lkp.afe.noise.eval = afe_config.lkp.noise; % Set LKP AFE noise model
  
  %%% Configure equalizers %%%
  act.dut.pma.ffe.b = zeros(1, eq_config.dut.ffe_length); % Initialize DUT FFE coefficients
  act.dut.pma.dfe.b = zeros(1, eq_config.dut.dfe_length); % Initialize DUT DFE coefficients
  act.lkp.pma.ffe.b = zeros(1, eq_config.lkp.ffe_length); % Initialize LKP FFE coefficients
  act.lkp.pma.dfe.b = zeros(1, eq_config.lkp.dfe_length); % Initialize LKP DFE coefficients

  %%% EQ "train", etc. %%%
  act = simDM_ACT_training(act, ldr_symbols); % Train the equalizers

  %%% Configure noise generator for the DUT and LKP %%%
  act.dut.pcb.noise.eval = pcb_config.noise; % Set DUT PCB noise model
  act.lkp.pcb.noise.eval = pcb_config.noise; % Set LKP PCB noise model

  %%% Run 5 rounds of 100 LDR symbols %%%
  for n = 1:5
    lkp_tx_symbol = round(rand(1, ldr_symbols)) * 2 - 1; % Generate LKP transmit symbols
    dut_tx_symbol = floor((pam_levels - 1e-9) * rand(1, act.ldr_oversampling * ldr_symbols / act.hdr_oversampling)) * 2 / (pam_levels - 1) - 1; % Adjust DUT transmit symbols for PAM levels
    act = link(act, lkp_tx_symbol, dut_tx_symbol); % Simulate the link
  end

end
