function act = simDM_ACT_training(act, ldr_symbols)
% simDM_ACT_training - Simulates the training process for the ACT 
%
% Syntax: act = simDM_ACT_training(act, ldr_symbols)
%
% Inputs:
%    act - Structure containing the state and parameters of the simulation.
%    ldr_symbols - Number of symbols for the LDR (Low Data Rate) line probe.
%
% Outputs:
%    act - Updated structure with the results of the training process.
%
% This is simulation code provided to help with the development of
% IEEE 802.3dm.
%
% This code is provided for reference to allow independent evaluation
% of the accuracy and applicability of the simulation results shared in
% IEEE 802.3dm presentations by the author.
%
% Written by Ragnar Jonsson, affiliated with Marvell Technology, Inc.
% Version 1.1.1, January 30th, 2025
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.

  %%% Run 4 rounds of LDR line probe %%%
  %%% Half-duplex probing with only LKP (switch side) transmitting %%%
  for n = 1:4
    %%% Generate LKP transmission symbols (random -1 or 1) %%%
    lkp_tx_symbol = round(rand(1, ldr_symbols)) * 2 - 1;
    %%% Generate DUT transmission symbols (zeros) %%%
    dut_tx_symbol = zeros(1, act.ldr_oversampling * ldr_symbols / act.hdr_oversampling);
    %%% Simulate the link with the generated symbols %%%
    act = link(act, lkp_tx_symbol, dut_tx_symbol);
  end
  %%% Store the received LDR probe signals %%%
  act.probe.lkp_ldr_probe = act.lkp.afe.rx_out;
  act.probe.dut_ldr_probe = act.dut.afe.rx_out;
  
  %%% Run 4 rounds of HDR line probe %%%
  %%% Half-duplex probing with only DUT (camera side) transmitting %%%
  %%% The probing signal is a series of impulses %%%
  tmp = act.lkp.afe.noise.eval;
  act.lkp.afe.noise.eval = 'state.out = 0;';
  for n = 1:4
    %%% Generate LKP transmission symbols (zeros) %%%
    lkp_tx_symbol = zeros(1, ldr_symbols);
    %%% Generate DUT transmission symbols (impulse at the beginning) %%%
    dut_tx_symbol = zeros(1, act.ldr_oversampling * ldr_symbols / act.hdr_oversampling);
    dut_tx_symbol(1) = 1;
    %%% Simulate the link with the generated symbols %%%
    act = link(act, lkp_tx_symbol, dut_tx_symbol);
  end
  act.lkp.afe.noise.eval = tmp;
  %%% Store the received HDR probe signals %%%
  act.probe.lkp_hdr_probe = act.lkp.afe.rx_out;
  act.probe.dut_hdr_probe = act.dut.afe.rx_out;
 
  %%% Extract equalizer training signal from probe signal %%%
  %%% Find the peak in the received HDR probe signal %%%
  [mx, ix] = max(abs(act.probe.lkp_hdr_probe(1:1000)));
  ix = floor(ix / 2) * 2 + 1;
  ix = max(1, ix - 10);
  %%% Extract the training signal starting from the peak %%%
  training_rx = act.probe.lkp_hdr_probe(ix:2000);
  act.probe.training_rx = training_rx;
  
  %%% Calculate equalizer coefficients %%%
  %%% Get the lengths of the FFE and DFE filters %%%
  ffe_length = length(act.lkp.pma.ffe.b);
  dfe_length = length(act.lkp.pma.dfe.b);
  %%% Set the equalizer delay %%%
  eq_delay = floor(ffe_length / 2);
  %%% Set the noise auto-correlation (ignored for ZFE) %%%
  R_noise = 0.001^2;

  %%% Calculate the MMSE equalizer coefficients %%%
  [ffe, dfe] = mmse_eq(training_rx, R_noise, eq_delay, ffe_length, dfe_length, act.lkp.pma.ffe_oversampling);
  %%% Update the FFE and DFE coefficients in the act structure %%%
  act.lkp.pma.ffe.b = ffe;
  act.lkp.pma.ffe.state = [];  % Reset FFE state
  act.lkp.pma.dfe.b = -dfe;
  act.lkp.pma.dfe.state = [];  % Reset DFE state
end
