function h = simDM_modulated_pulse(t, t_r, t_d, A, f_1, t_interval)
% Function to generate modulated pulse (transient noise) signal
% Arguments:
%   t - time vector
%   t_r - rise time constant in ns
%   t_d - decay time constant in ns
%   A - amplitude of the transient noise in mV
%   f_1 - frequency of the tone in MHz (default is 0)
%   t_interval - time interval for modulation in ms (default is new transient every call)
% Returns:
%   h - modulated pulse signal
% Zero t_r means that the pulse becaumes a square pulse
% If t_interval is not provided, a new transient is generated every call
% Usage:
%   h = simDM_modulated_pulse(t, t_r, t_d, A, f_1, t_interval)
% Example:
%   t = [0:1000]*1e9; % Time vector in ns
%   h = simDM_modulated_pulse(t, 10, 100, 100, 10);
%   plot(t, h);
%   xlabel('Time [ns]');
%   ylabel('Amplitude [mV]');
%   title('Modulated Pulse Signal');
%   grid on;
% This will plot modulated pulse with rise time of 10ns, decay time of 100ns,
% amplitude of 100mV, and tone frequency of 10MHz.
%
% This is simulation code provided to help with the development of
% IEEE 802.3dm.
%
% This code is provided for reference to allow independent evaluation
% of the accuracy and applicability of the simulation results shared in
% IEEE 802.3dm presentations by the author.
%
% Written by Ragnar Jonsson, affiliated with Marvell Technology, Inc.
% Version 1.1.1, February 24th, 2025
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.

    % Set default frequency if not provided
    if ((nargin < 5) || isempty(f_1))
        f_1 = 0;
    end

    % Set default time interval if not provided
    if ((nargin < 6) || isempty(t_interval))
        t_interval = t(1)*1e9 + 100; % Default to new transient 100ns into every call
    end

    % change argument units
    t_r = t_r * 1e-9; % Convert rise time to seconds
    t_d = t_d * 1e-9; % Convert decay time to seconds
    t_interval = t_interval * 1e-9; % Convert time interval to seconds
    A = A * 1e-3; % Convert amplitude to V
    f_1 = f_1 *1e6; % Convert frequency to MHz

    % Modulate time vector by the time interval
    t = mod(t, t_interval);

    % Generate the pulse shape
    if(t_r == 0)
        pulse = (t <= t_d); % Generate a square pulse
	else
        % Calculate rise and decay constants
        a_r = log(0.1 * 2.45) / t_r;
        a_d = log(0.1 / 1.35) / t_d;

        % Generate the rising and decaying pulse shape
        h = ((1 - exp(a_r * t)) .* exp(a_d * t));
        pulse = h ./ max(h(:)); % Normalize the pulse
    end

    % Generate the tone
    tone = cos(2 * pi * t * f_1);

    % Combine pulse and tone to create transient noise
    transient = A * (pulse .* tone);
    h = transient; % Output the transient noise signal
end

